/*
 Luminardo with I2C 128x64 SSD1306
 
 Copyright (c) 2015 Dmitry Pakhomenko.
 dmitryp@magictale.com
 http://magictale.com
 
 This example code is in the public domain.
 */
 
//#define BLINK_ENABLED

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <glcdfont.h>
#include "icons.h"

//PB0 - not actually connected anywhere as our OLED doesn't have RES
#define OLED_RESET 0
Adafruit_SSD1306 display(OLED_RESET);

#if (SSD1306_LCDHEIGHT != 64)
#error("Height incorrect, please fix Adafruit_SSD1306.h!");
#endif

void drawBitmapCentred(const uint8_t *bitmap, uint16_t color, bool _blink = false)
{
    uint8_t b_width = pgm_read_byte(bitmap++);
    uint8_t b_height = pgm_read_byte(bitmap++);
    display.drawBitmap((display.width() - b_width) / 2, (display.height() - b_height) / 2, bitmap, b_width, b_height, color, _blink);
}

void counter(uint16_t spd)
{
  display.clearDisplay();
  display.setTextColor(WHITE, BLACK);
  
  display.setTextSize(1);
  display.setFont(sevenSegNumFont, 0);
  display.setCursor(0, 0);
  display.print(spd, DEC);  
  
  display.display();
}

void setup()   
{
  Serial.begin(57600);
  Serial.println("Luminardo SSD1306 I2C display test started");

  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // 0x3D initialize with the I2C addr 0x3D (for the 128x64)
  
  display.display(); // show splashscreen
  delay(3000);

  display.dim(true);
  delay(1000);
  display.dim(false);  
  display.clearDisplay();   // clears the screen and buffer

  display.setFont(smallFont, 1, true);
  display.setTextSize(1);
  display.setTextColor(WHITE, BLACK);
  display.setCursor(0,30);
  display.println("http://magictale.com");
  display.display();
  
  for (uint8_t i = 0; i < 10; i++)
  {
    delay(500);
    display.flipBuffer();
  }

  display.clearDisplay();   // clears the screen and buffer  
  display.setTextSize(1);
  display.setTextColor(WHITE, BLACK);
  display.setCursor(0, 0);  
  display.setFont(smallFont, 1);
  display.print("Small ");
  display.setFont(smallFont, 3);
  display.println("Small");
  
  display.setFont(mediumFont, 1);
  display.print("Medium ");
  display.setFont(mediumFont, 2);
  display.println("Medium");
  
  display.setFont(bigFont, 1);
  display.print("Big ");
  display.setFont(bigFont, 2, true);
  display.println("Big");
  
  display.display();
  
  for (uint8_t i = 0; i < 10; i++)
  {
    delay(500);
    display.flipBuffer();
  }

  display.clearDisplay();
  drawBitmapCentred(WiFi1, 1);
  display.display();
  delay(2000);
  
  display.clearDisplay();
  drawBitmapCentred(WiFi2, 1);
  display.display();
  delay(2000);
   
  display.clearDisplay();
  drawBitmapCentred(WashingMachine, 1);
  display.display();
  delay(2000);

  display.clearDisplay();
  drawBitmapCentred(Clock, 1);
  display.display();
  delay(2000);
  
  display.clearDisplay();
  drawBitmapCentred(CarBat, 1, true);
  display.display();
  
  for (uint8_t i = 0; i < 10; i++)
  {
    delay(500);
    display.flipBuffer();
  }

  
  uint16_t spd = 0;
  while(spd < 999)
  {
    counter(spd);
    delay(100);  
    spd++;
  }

}

void loop() 
{
  
}
