/*
 Luminardo OBDII electronic gauge
 with I2C 128x64 SSD1306 OLED display
 
 Copyright (c) 2015 Dmitry Pakhomenko.
 dmitryp@magictale.com
 http://magictale.com/luminardo
 
 This example code is in the public domain.
 */

#include <Board.h>                         //Generic Luminardo definitions
#include <OneWire.h>                       //OneWire temperature sensor 
#include <SPI.h>                           //Here and below - for OLED display support
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <glcdfont.h>
#include <SPPi.h>                          //Bluetooth SPP
#include <usbhub.h>                        //USB host lib
#include "Icons.h"                         //Icons and pictures
#include "Messages.h"                      //Collection of string constants
#include "Utils.h"                         //Conversion utils and helpers
#include "Luminardo_OBDII_BT_128x64_i2c.h" //Application specific definitions

//PB0 - not actually connected anywhere as our OLED doesn't have RES
#define OLED_RESET 0
Adafruit_SSD1306 display(OLED_RESET);
#if (SSD1306_LCDHEIGHT != 64)
#error("Incorrect height, this sketch works with 128x64 resolution only");
#endif

USB Usb;
BTD Btd(&Usb);
//40:22:11:00:69:58 OBDII
uint8_t remote_addr[6] = {0x58, 0x69, 0x00, 0x11, 0x22, 0x40};

SPPi SerialBT(&Btd, "Luminardo", "1234", true, (uint8_t*)&remote_addr);
boolean firstMessage = true;

void disp_bt_status(bool flash)
{
  uint8_t x_pos = pgm_read_byte(ThermometerSmall) + 1;  
  drawBitmap(x_pos + 2 * pgm_read_byte(sevenSegNumSmallFont+1) + 17, 0, BTSmall2, 1, flash);    
}

void disp_temperature()
{
  uint8_t x_pos = pgm_read_byte(ThermometerSmall) + 1;
  display.setCursor(x_pos - 3, 0);

  display.setTextColor(WHITE);  
  display.setFont(bigFont, 1);  
  if (temperature > 0) display.write('+');
  else if (temperature < 0) display.write('-');

  display.setCursor(x_pos + 11, 0);
  display.setTextColor(WHITE, BLACK);
  display.setFont(sevenSegNumSmallFont, 1);
  display.setTextSize(1);
  display.print((long)temperature, DEC);  
}

void disp_tank_level()
{
  uint8_t x_pos = pgm_read_byte(FuelTankSmall);  
  display.setFont(sevenSegNumSmallFont, 1);  
  display.setCursor(x_pos + 1, 23);
  display.print(tank_level, DEC);
}

void disp_bat_voltage()
{
  uint8_t y_pos =  display.height() - pgm_read_byte(CarBatSmall+1);
  uint8_t x_pos = pgm_read_byte(CarBatSmall);
  display.setFont(sevenSegNumSmallFont, 1);  
  
  if (bat_voltage == 0)
  {
    display.setCursor(x_pos + 1, y_pos);
    display.print(bat_voltage, DEC);
    display.print(F("  "));
  }else{
    display.setCursor(x_pos - 4, y_pos);
    display.print(bat_voltage / 10, DEC);
    
    x_pos = x_pos + pgm_read_byte(sevenSegNumSmallFont + 1) * 2 + 3;
    display.setCursor(x_pos, y_pos);    
    
    display.drawPixel(x_pos - 3, y_pos + 15, WHITE);
    display.drawPixel(x_pos - 2, y_pos + 15, WHITE);
    display.drawPixel(x_pos - 3, y_pos + 14, WHITE);
    display.drawPixel(x_pos - 2, y_pos + 14, WHITE);
    
    display.print(bat_voltage - (bat_voltage / 10 * 10), DEC);
  }
  
  y_pos =  display.height() - pgm_read_byte(CarBatSmall+1);
  
  if (bat_voltage <= BATTERY_VOLTAGE_WARN_THRESHOLD)
  {
    display.fillRect(0, y_pos, pgm_read_byte(CarBatSmall), pgm_read_byte(CarBatSmall+1), BLACK);
  }
  drawBitmap(0, y_pos, CarBatSmall, 1, bat_voltage <= BATTERY_VOLTAGE_WARN_THRESHOLD);
}

void disp_speed()
{
  char buffer[4] = "";  

  display.setFont(sevenSegNumMediumFont, 0);
  uint8_t f_width = pgm_read_byte(sevenSegNumMediumFont+1);  
  uint8_t f_height = pgm_read_byte(sevenSegNumMediumFont);  
  uint8_t x_pos = display.width() - 3 * f_width;
  uint8_t y_pos = display.height() - f_height;
  display.setCursor(x_pos, y_pos);
  sprintf(buffer, "%3d", speed);   
  display.print(buffer);

  display.setFont(mediumFont, 0);  
  x_pos -= 1 * pgm_read_byte(mediumFont);
  display.setCursor(y_pos - 20, x_pos + 15);
  display.setRotation(3);
  oled_print_p(KMH);
  display.setRotation(0);
}

void disp_rpm()
{
  char buffer[5] = "";    

  display.setFont(sevenSegNumSmallFont, 1);
  uint8_t f_width = pgm_read_byte(sevenSegNumSmallFont+1);  
  uint8_t f_height = pgm_read_byte(sevenSegNumSmallFont);  
  uint8_t x_pos = display.width() - 4 * (f_width + 1);

  display.setCursor(x_pos, 0);
  sprintf(buffer, "%4d", rpm);   
  display.print(buffer);

  display.setFont(mediumFont, 0);  
  x_pos -= 3 * pgm_read_byte(mediumFont);
  display.setCursor(x_pos , 3);
  oled_print_p(RPM); 
}

void disp_dashboard_full()
{
  uint8_t f_width;
  uint8_t f_height;  
  uint8_t x_pos, y_pos;
  
  display.clearDisplay();
  display.setTextColor(WHITE, BLACK);  
  
  drawBitmap(0, 0, ThermometerSmall, 1);  
  disp_bt_status(false);

  drawBitmap(0, 23, FuelTankSmall, 1);
  disp_tank_level();  

  disp_bat_voltage();
 
  disp_rpm();
 
  disp_speed();
 
  display.display();  
}

bool get_bt_state()
{
  if (SerialBT.connected) 
  {
    if (firstMessage) 
    {
      serial_print_p(BT_CONNECTED);
      Serial.println();
      firstMessage = false;
      
      disp_bt_status(false);
      bt_state = btConnected;
    }
#ifdef ELM_CONSOLE_BI_REDIRECT    
    if (Serial.available())
      SerialBT.write(Serial.read());
    if (SerialBT.available())
      Serial.write(SerialBT.read());
#endif
    return true;
  }
  else
  {
    firstMessage = true;
    if (bt_state != btDisconnected)
    {
      disp_bt_status(true);
      bt_state = btDisconnected;
    }
    return false;
  }
}

float get_temperature(uint8_t* data)
{
  int HighByte, LowByte, TReading, SignBit, Tc_100;
  float res;

  LowByte = data[0];
  HighByte = data[1];
  TReading = (HighByte << 8) + LowByte;
  SignBit = TReading & 0x8000;  // test most sig bit
  if (SignBit) // negative
    TReading = (TReading ^ 0xffff) + 1; // 2's comp

  Tc_100 = (6 * TReading) + TReading / 4;    // multiply by (100 * 0.0625) or 6.25

  res = Tc_100 / 100;
  if (SignBit) res *= -1;

  return res;
}

bool read_temperature()
{
  bool res = false;
  if (dsState == dsIdle)
  {
    if ( ds.search(dsAddr)) 
    {
      if (ds.crc8(dsAddr, 7) == dsAddr[7])
      {
        if (dsAddr[0] == 0x28) 
        {
          ds.reset();
          ds.select(dsAddr);
          ds.write(0x44);
          dsState = dsTempConversion;
        }
      }
    }
    else 
    {
      ds.reset_search();
      dsState = dsInitDelay;
    }
  }
  else if (dsState == dsTempConversion)
  {
    ds.reset();
    ds.select(dsAddr);
    ds.write(0xBE);
    for (uint8_t i = 0; i < 9; i++) 
      dsData[i] = ds.read();

    if (ds.crc8(dsData, 8) == dsData[8])
    {
      temperature = get_temperature((uint8_t*)&dsData);
      
      if (temperature != FAILED_TEMPERATURE)
      {
        if (temperature > 0) Serial.write('+');
        else if (temperature < 0) Serial.write('-');
        Serial.print((long)temperature, DEC);
        Serial.write('C');
        Serial.println();        
        res = true;
      }
    }
    dsState = dsIdle;
  }
  else if (dsState == dsInitDelay)
  {
    dsState = dsIdle;
  }
  return res;
}

bool read_battery_voltage()
{
  if (serial_bt_request_response(ELM_READ_SYS_VOLTAGE, 2, DEF_BT_CMD_TIMEOUT) != 5) return false;
  else if (rxData[2] != '.') return false;
  else if (rxData[4] != 'V') return false;
  
  char* nextChar;
  bat_voltage = strtol(rxData, &nextChar, 10) * 10;
  bat_voltage += strtol(&rxData[3], &nextChar, 10);
  
  return true;
}

bool read_rpm()
{
  if (serial_bt_request_response(ELM_READ_ENGINE_RPM, 2, DEF_BT_CMD_TIMEOUT) != 12) return false;
  rpm = ((strtol(&rxData[6], 0, 16) * 256) + strtol(&rxData[9], 0, 16)) / 4;   
  return true;
}

bool read_speed()
{
  if (serial_bt_request_response(ELM_READ_VEHICLE_SPEED, 2, DEF_BT_CMD_TIMEOUT) != 9) return false;
  speed = strtol(&rxData[6], 0, 16);
  return true;
}

bool read_tank_level()
{
  if (serial_bt_request_response(ELM_READ_FUEL_LEVEL, 2, DEF_BT_CMD_TIMEOUT) != 9) return false;
  tank_level = strtol(&rxData[6], 0, 16);
  return true;
}

//Standard Arduino routines
void setup()   
{
  Serial.begin(57600);
  serial_print_p(PRODUCT_NAME);
  serial_println_p(STARTED);
  
  if (Usb.Init() == -1) {
    serial_println_p(OSC_FAILED);
    while (1); //halt
  }
  serial_print_p(SPP_BT_LIB);
  serial_println_p(STARTED);
   
  serial_print_p(ENABLE_VBUS); 
  Usb.gpioWr(1);
  serial_println_p(ENABLED);

  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3D (for the 128x64)
  display.display(); // show splashscreen
  
  last_time_check = millis();
  last_time_disp_flip = millis();  
  last_temp_refresh = 0; //trigger immediate refresh
  last_tank_level_refresh = 0; //trigger immediate refresh
  last_bat_voltage_refresh = 0; //trigger immediate refresh
  
  sys_state = sysSplashScreen;
  bt_state = btDisconnected;
  
  //Initialize temperature sensor variables
  temperature = FAILED_TEMPERATURE;
  dsState = dsIdle;

  tank_level = 0;
  bat_voltage = 0;
  speed = 0;
  rpm = 0;
}


void loop() 
{
  //Handle display flashing
  if (millis() - last_time_disp_flip > DISP_FLIP_PEDIOD)
  {
    last_time_disp_flip = millis();   
    display.flipBuffer();
  }
 
  Usb.Task(); //Do USB + BT routine

  switch (sys_state)
  {
      case sysSplashScreen:
        if (millis() - last_time_check > SPLASHSCREEN_TIMEOUT)
        {
          display.clearDisplay();  
          display.setTextColor(WHITE, BLACK);    
          drawBitmap(2, 0, LuminardoOBDIILogo, 1);
          display.setTextSize(1);    
          display.setFont(smallFont, 1);
          display.setCursor(0,55);
          oled_print_p(MAGICTALE_URL);
          display.display();
          sys_state = sysLogo;
        }        
      
      case sysLogo:    
        if (millis() - last_time_check > LOGO_TIMEOUT)
        {
          display.clearDisplay();
          drawBitmapCentred(BTLarge, 1, true);
          display.display();
          sys_state = sysStarting;
        }        
        break;
    
      case sysStarting:
        if (get_bt_state())
        {
            serial_bt_request_response(ELM_INIT, 4, DEF_BT_CMD_TIMEOUT);
            display.clearDisplay();
            display.setTextColor(WHITE, BLACK);    
            display.setTextSize(1);    
            display.setFont(mediumFont, 0);
            display.setCursor(10, 20);
            oled_print_p(CONNECTED_TO);            
            display.setCursor(0, 40);            
            display.print(rxData);
            display.display();
            
            last_time_check = millis();
            sys_state = sysCheckingOBDII;   
        }
        break;
        
      case sysCheckingOBDII:
        if (millis() - last_time_check > LOGO_TIMEOUT)
        {
          const char error_msg[] = "UNABLE TO CONNECT";
          uint8_t found = serial_bt_request_response(ELM_READ_ENGINE_RPM, 3, TEN_SEC_BT_CMD_TIMEOUT, error_msg);
          
#ifdef TEST_CONNECTION_2_VEHICLE          
          if (found == BT_REPLY_SUBSTR_FOUND)
          {
            display.clearDisplay();
            display.setTextColor(WHITE, BLACK);    
            display.setTextSize(1);    
            display.setFont(bigFont, 0, true);
            display.setCursor(5, 0);
            oled_print_p(VEHICLE);
            display.setFont(mediumFont, 0, true);
            display.setCursor(37, 16);
            oled_print_p(IS_NOT);
            display.setCursor(23, 28);
            oled_print_p(RESPONDING);            
            display.setFont(mediumFont, 0);
            display.setCursor(0, 40);            
            oled_print_p(OBDII_NOT_SUPPORTED);
            display.display();
            
            sys_state = sysFailure;
          }else
#endif          
          {
            if (read_battery_voltage() && bat_voltage <= BATTERY_VOLTAGE_WARN_THRESHOLD)
            {
              display.clearDisplay();
              drawBitmapCentred(CarBatBig, 1, true);
              display.display();
              last_time_check = millis();
              sys_state = sysBatWarning;
            }else
            {
              disp_dashboard_full();  
              sys_state = sysScreenOne;
            }
          }
        }      
        break;
        
      case sysBatWarning:
        if (millis() - last_time_check > LOGO_TIMEOUT)
        {
          if (read_tank_level() && tank_level <= TANK_LEVEL_WARN_THRESHOLD)
          {
            display.clearDisplay();
            drawBitmapCentred(FuelTankLarge, 1, true);
            display.display();
            last_time_check = millis();
            sys_state = sysFuelWarning;
          }else
          {
            disp_dashboard_full();  
            sys_state = sysScreenOne;
          }
        }
      
        break;
        
      case sysFuelWarning:
        if (millis() - last_time_check > LOGO_TIMEOUT)
        {
            disp_dashboard_full();  
            sys_state = sysScreenOne;
        }
        break;
        
      case sysScreenOne:
        if (millis() - last_temp_refresh > TEMPERATURE_REFRESH)
        {
          last_temp_refresh = millis();
          if (read_temperature()) disp_temperature();
        }
        
        if (millis() - last_bat_voltage_refresh > BAT_VOLTAGE_REFRESH)
        {
          last_bat_voltage_refresh = millis();
          if (read_battery_voltage()) disp_bat_voltage();
        }
        
        if (read_rpm()) disp_rpm();
        
        if (read_speed()) disp_speed();
        
        if (millis() - last_tank_level_refresh > TANK_LEVEL_REFRESH)
        {
          last_tank_level_refresh = millis();
          if (read_tank_level()) disp_tank_level();
        }
        
        break;
        
      case sysScreenTwo:
      
        break;
        
      case sysScreenThree:
      
        break;
        
      case sysFailure:
      
        break;
        
      default:
      
        break;
    
  }
  delay(100);
}

