/*
 Luminardo OBDII electronic gauge
 with I2C 128x64 SSD1306 OLED display
 
 Copyright (c) 2015 Dmitry Pakhomenko.
 dmitryp@magictale.com
 http://magictale.com/luminardo
 
 This example code is in the public domain.
 */

#ifndef Utils_h
#define Utils_h

#include "Luminardo_OBDII_BT_128x64_i2c.h" //Application specific definitions

extern Adafruit_SSD1306 display;
extern SPPi SerialBT;
extern USB Usb;
extern char rxData[ELM_RX_BUF_SIZE];
extern char rxIndex;

void serial_print_p(const prog_char str[])
{
  char c;
  if(!str) return;
  while((c = pgm_read_byte(str++)))
    Serial.write(c);
}

void serial_println_p(const prog_char str[])
{
  serial_print_p(str);
  Serial.println();
}

void oled_print_p(const prog_char str[])
{
  char c;
  if(!str) return;
  while((c = pgm_read_byte(str++)))
    display.write(c);
}

void serial_bt_print_p(const prog_char str[])
{
  char c;
  if(!str) return;
  while((c = pgm_read_byte(str++)))
    SerialBT.write(c);
}

void serial_bt_println_p(const prog_char str[])
{
  serial_bt_print_p(str);
  SerialBT.println();
}

uint8_t serial_bt_get_response(uint8_t line, unsigned long timeout, const char search[] = NULL)
{
  char inChar;
  rxIndex = 0;    
  unsigned long start_time = millis();
  unsigned long end_time = start_time;

  while (rxIndex == 0 || line != 0 && (end_time - start_time < timeout))
  {
    if (SerialBT.available() > 0)
    {
      inChar = SerialBT.read();
      if (inChar == '\r')
      {        
        line--;
        rxData[rxIndex] = '\0';
        
        if (search != NULL)
        {
          char* found = strstr(rxData, search); 
          if (found != NULL) return BT_REPLY_SUBSTR_FOUND;
        }
        
        if (line != 0) rxIndex = 0;
        else return rxIndex;
      }
      else
      {
        if (rxIndex >= ELM_RX_BUF_SIZE) 
        {
          return rxIndex; //sanity check
        }
        rxData[rxIndex++] = inChar;
      }      
    }else
    {
      Usb.Task();
    }
    end_time = millis();
  }
  return rxIndex;
}

uint8_t serial_bt_request_response(const prog_char cmd[], uint8_t line_num, unsigned long timeout, const char search[] = NULL)
{
  serial_bt_println_p(cmd);
  SerialBT.send();
  uint8_t len = serial_bt_get_response(line_num, timeout, search);
  serial_print_p(ELM_PREFIX);
  Serial.print(len, DEC);
  serial_print_p(ELM_SUFFIX);
  Serial.println(rxData);
  //dump remaining data to get ready for the next round
  while (SerialBT.available() > 0) SerialBT.read();  
  return len;
}

void reverse_bit_order(const uint8_t *buf, size_t sz)
{
    uint8_t cntr = 0;
    
    for (size_t i = 0; i < sz; i++)
    {
        uint8_t bt = pgm_read_byte(buf + i);
        uint8_t res = 0;
        for (uint8_t j = 0; j < 8; j++)
        {
            res = res << 1;
            res = res | ((bt >> j) & 1);
        }  

        Serial.print("0x");        
        if (res < 16) Serial.print("0");
        Serial.print(res, HEX);
        Serial.print(", ");                
        
        cntr++;
        if (cntr == 200)
        {
            Serial.println();
            cntr = 0;  
        }
    }
}

void drawBitmapCentred(const uint8_t *bitmap, uint16_t color, bool _blink = false)
{
    uint8_t b_width = pgm_read_byte(bitmap++);
    uint8_t b_height = pgm_read_byte(bitmap++);
    display.drawBitmap((display.width() - b_width) / 2, (display.height() - b_height) / 2, bitmap, b_width, b_height, color, _blink);
}

void drawBitmap(int16_t x, int16_t y, const uint8_t *bitmap, uint16_t color, bool _blink = false)
{
    uint8_t b_width = pgm_read_byte(bitmap++);
    uint8_t b_height = pgm_read_byte(bitmap++);
    display.drawBitmap(x, y, bitmap, b_width, b_height, color, _blink);
}


#endif


